﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Security.Cryptography;
using System.IO;

/*
 *         Progetto: Cifratura/decifratura DES mediante FileStream
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 16/02/2015
 *  
 *      Descrizione: Cifratura e decifratura di file mediante algoritmo DES.
 */

namespace DESMemoria
{
    class Program
    {
        // Vettori di inizializzazione usati per derivare la chiave a partire
        // da una password testuale o per applicare l'agoritmo. 
        // Utilizzate in entrambe le fasi di cifratura e decifratura.

        static readonly byte[] IVPassword = new byte[8] { 10, 02, 20, 16, 10, 48, 00, 00 };
        static readonly byte[] IVAlgoritmo = new byte[8] { 10, 02, 20, 16, 10, 56, 00, 00 };

        static void CifraDES()
        {
            string plainFile, cipherFile;
            string pw;
            Console.Clear();
            do
            {
                Console.Write("Nome del file da cifrare: ");
                plainFile = Console.ReadLine().Trim();
            }
            while (plainFile.Length == 0);
            do
            {
                Console.Write("Nome del file di output: ");
                cipherFile = Console.ReadLine().Trim();
            }
            while (cipherFile.Length == 0);

            
            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave DES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new PasswordDeriveBytes(pw, null);
            byte[] k = pdb.CryptDeriveKey("DES", null, 0, IVPassword);

            var csp = new DESCryptoServiceProvider();
            var enc = csp.CreateEncryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new FileStream(plainFile, FileMode.Open))
            using (var ostream = new FileStream(cipherFile, FileMode.Create))
            {

                using (var cstream = new CryptoStream(ostream, enc, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }
            }

            Console.WriteLine();
            Console.WriteLine("Documento cifrato e memorizzato in {0}", cipherFile);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();
        }


        static void DecifraDES()
        {
            string plainFile, cipherFile;
            string pw;
            Console.Clear();
            do
            {
                Console.Write("Nome del file da decifrare: ");
                cipherFile = Console.ReadLine().Trim();
            }
            while (cipherFile.Length == 0);
            do
            {
                Console.Write("Nome del file di output: ");
                plainFile = Console.ReadLine().Trim();
            }
            while (plainFile.Length == 0);


            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave DES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new PasswordDeriveBytes(pw, null);
            byte[] k = pdb.CryptDeriveKey("DES", null, 0, IVPassword);

            var csp = new DESCryptoServiceProvider();
            var enc = csp.CreateDecryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new FileStream(cipherFile, FileMode.Open))
            using (var ostream = new FileStream(plainFile, FileMode.Create))
            {

                using (var cstream = new CryptoStream(ostream, enc, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }
            }

            Console.WriteLine();
            Console.WriteLine("Documento decifrato e memorizzato in {0}", plainFile);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();
        }

        static void Main(string[] args)
        {
            int scelta;

            do
            {
                Console.Clear();
                Console.WriteLine("Algoritmo di cifratura DES mediante FileStream");
                Console.WriteLine();
                Console.WriteLine("1. Cifra file");
                Console.WriteLine("2. Decifra file");
                Console.WriteLine("0. Uscita");
                Console.WriteLine();
                Console.Write("Scelta: ");


                if (!Int32.TryParse(Console.ReadLine(), out scelta))
                    scelta = -1;

                try
                {
                    switch (scelta)
                    {
                        case 1:
                            CifraDES();
                            break;
                        case 2:
                            DecifraDES();
                            break;
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine();
                    Console.WriteLine(String.Format("Eccezione: {0}", ex.Message));
                    Console.WriteLine();
                    Console.Write("Premere un tasto per continuare...");
                    Console.ReadKey();
                }
            }
            while (scelta != 0);
        }
    }
}
