﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Security.Cryptography;
using System.IO;

/*
 *         Progetto: Cifratura/decifratura DES mediante MemoryStream
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 16/02/2015
 *  
 *      Descrizione: Cifratura e decifratura di stringhe mediante algoritmo DES.
 */

namespace DESMemoria
{
    class Program
    {
        // Vettori di inizializzazione usati per derivare la chiave a partire
        // da una password testuale o per applicare l'agoritmo. 
        // Utilizzate in entrambe le fasi di cifratura e decifratura.

        static readonly byte[] IVPassword = new byte[8] { 10, 02, 20, 16, 10, 48, 00, 00 };
        static readonly byte[] IVAlgoritmo = new byte[8] { 10, 02, 20, 16, 10, 56, 00, 00 };

        static void CifraDES()
        {
            string plainText, cipherText;
            string pw;
            Console.Clear();
            Console.Write("Stringa Unicode da cifrare: ");
            plainText = Console.ReadLine();
            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave DES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new PasswordDeriveBytes(pw, null);
            byte[] k = pdb.CryptDeriveKey("DES", null, 0, IVPassword);

            var csp = new DESCryptoServiceProvider();
            var enc = csp.CreateEncryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new MemoryStream(Encoding.UTF8.GetBytes(plainText)))
            using (var ostream = new MemoryStream())
            {

                using (var cstream = new CryptoStream(ostream, enc, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }

                cipherText = Convert.ToBase64String(ostream.ToArray());
            }

            Console.WriteLine();
            Console.WriteLine("Messaggio cifrato (Base64): {0}", cipherText);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();
        }


        static void DecifraDES()
        {
            string plainText, cipherText;
            string pw;
            Console.Clear();
            Console.Write("Stringa Base64 da decifrare: ");
            cipherText = Console.ReadLine();
            Console.Write("Password: ");
            pw = Console.ReadLine();

            // Genera una chiave DES di lunghezza apposita a partire
            // dalla password inserita dall'utente
            var pdb = new PasswordDeriveBytes(pw, null);

            byte[] k = pdb.CryptDeriveKey("DES", null, 0, IVPassword);
            var csp = new DESCryptoServiceProvider();


            var enc = csp.CreateDecryptor(k, IVAlgoritmo);

            Console.WriteLine("Lunghezza della chiave: {0} bit", k.Length * 8);

            using (var istream = new MemoryStream(Convert.FromBase64String(cipherText)))
            using (var ostream = new MemoryStream())
            {

                using (var cstream = new CryptoStream(ostream, enc, CryptoStreamMode.Write))
                {
                    istream.CopyTo(cstream);
                }

                plainText = Encoding.UTF8.GetString(ostream.ToArray());
            }

            Console.WriteLine();
            Console.WriteLine("Messaggio Unicode decifrato: {0}", plainText);
            Console.WriteLine();
            Console.Write("Premere un tasto per continuare...");

            Console.ReadKey();

        }
        static void Main(string[] args)
        {
            int scelta;

            do
            {
                Console.Clear();
                Console.WriteLine("Algoritmo di cifratura DES mediante MemoryStream");
                Console.WriteLine();
                Console.WriteLine("1. Cifra");
                Console.WriteLine("2. Decifra");
                Console.WriteLine("0. Uscita");
                Console.WriteLine();
                Console.Write("Scelta: ");


                if (!Int32.TryParse(Console.ReadLine(), out scelta))
                    scelta = -1;

                try
                {
                    switch (scelta)
                    {
                        case 1:
                            CifraDES();
                            break;
                        case 2:
                            DecifraDES();
                            break;
                    }
                }
                catch (Exception ex)
                {
                    Console.WriteLine();
                    Console.WriteLine(String.Format("Eccezione: {0}", ex.Message));
                    Console.WriteLine();
                    Console.Write("Premere un tasto per continuare...");
                    Console.ReadKey();
                }
            }
            while (scelta != 0);
        }
    }
}
